import datetime
import re

from cloud_governance.cloud_resource_orchestration.utils.common_operations import string_equal_ignore_case
from cloud_governance.policy.helpers.aws.aws_policy_operations import AWSPolicyOperations


class ClusterRun(AWSPolicyOperations):
    """
    This class performs the operations on running cluster resources
    """

    def __init__(self):
        super().__init__()

    def run_policy_operations(self):
        """
        This method returns the running vms in the AAzure cloud and stops based on the action
        :return:
        :rtype:
        """
        instances = self._get_all_instances()
        cluster_data = {}
        for instance in instances:
            tags = instance.get('Tags', [])
            cluster_tag = self._get_cluster_tag(tags=tags).strip()
            instance_state = instance.get('State', {}).get('Name')
            if cluster_tag and not string_equal_ignore_case(instance_state, 'terminated'):
                launch_time = instance.get('LaunchTime')
                running_instances = stopped_instances = 0
                running_days = self.calculate_days(instance.get('LaunchTime'))
                if string_equal_ignore_case(instance_state, 'stopped'):
                    stopped_instances = 1
                    state_transition_reason = instance.get('StateTransitionReason')
                    if state_transition_reason:
                        extract_data = re.search(r'\((\d{4}-\d{2}-\d{2})', state_transition_reason)
                        if extract_data:
                            running_days = self.calculate_days(extract_data.group(1).split()[0], start_date=launch_time)
                            instance_state += f"@{extract_data.group(1)}"
                else:
                    running_instances = 1
                instance_data = f"{instance.get('InstanceId')}, {self.get_tag_name_from_tags(tags=tags, tag_name='Name')}, {instance.get('InstanceType')}, {instance_state}, {running_days}, {launch_time}"
                if cluster_tag in cluster_data:
                    cluster_data[cluster_tag]['Instances'].append(instance_data)
                    cluster_data[cluster_tag]['InstanceCount'] = len(cluster_data[cluster_tag]['Instances'])
                    cluster_data[cluster_tag]['Stopped'] = int(cluster_data[cluster_tag]['Stopped']) + stopped_instances
                    cluster_data[cluster_tag]['Running'] = int(cluster_data[cluster_tag]['Running']) + running_instances
                else:
                    cluster_data[cluster_tag] = {
                        'ResourceId': cluster_tag,
                        'ClusterTag': cluster_tag,
                        'User': self.get_tag_name_from_tags(tags=tags, tag_name='User'),
                        'RunningDays': running_days,
                        'RegionName': self._region,
                        'PublicCloud': self._cloud_name,
                        'Instances': [instance_data],
                        'InstanceCount': 1,
                        'Stopped': stopped_instances,
                        'Running': running_instances,
                        'index-id': f'{datetime.datetime.utcnow().date()}-{self._cloud_name.lower()}-{self.account.lower()}-{self._region.lower()}-{cluster_tag}'
                    }
        return list(cluster_data.values())



import asyncio

import aiohttp
import requests

from cloud_governance.common.logger.init_logger import logger


class APIRequests:

    def __init__(self):
        self.__loop = asyncio.new_event_loop()

    def get(self, url: str, **kwargs):
        try:
            response = requests.get(url, **kwargs)
            if response.ok:
                return response.json()
            else:
                return response.text
        except Exception as err:
            raise err

    def post(self, url: str,  **kwargs):
        try:
            response = requests.post(url, **kwargs)
            return response
        except Exception as err:
            raise err



from azure.core.exceptions import HttpResponseError
from azure.mgmt.network import NetworkManagementClient

from cloud_governance.common.clouds.azure.common.common_operations import CommonOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.utils.utils import Utils


class NetworkOperations(CommonOperations):

    def __init__(self):
        super().__init__()
        self.__network_client = NetworkManagementClient(self._default_creds, subscription_id=self._subscription_id)

    def __get_all_public_ip_address(self):
        """
        This method returns all the Public address
        :return:
        :rtype:
        """
        ips = self.__network_client.public_ip_addresses.list_all()
        return self._item_paged_iterator(item_paged_object=ips, as_dict=True)

    def get_public_ipv4_addresses(self):
        """
        This method returns the Public IPV4
        :return:
        :rtype:
        """
        public_addresses = []
        for ipaddress in self.__get_all_public_ip_address():
            if Utils.equal_ignore_case(ipaddress.get('public_ip_address_version'), 'IPv4'):
                if Utils.equal_ignore_case(ipaddress.get('public_ip_allocation_method'), 'Static'):
                    public_addresses.append(ipaddress)
        return public_addresses

    def __get_network_interfaces(self):
        """
        This method returns the network interfaces
        :return:
        :rtype:
        """
        network_interfaces = self.__network_client.network_interfaces.list_all()
        network_interfaces = self._item_paged_iterator(item_paged_object=network_interfaces, as_dict=True)
        return network_interfaces

    def get_public_ipv4_network_interfaces(self):
        """
        This method returns the network interfaces have the public ip address attached
        :return:
        :rtype:
        """
        public_ipv4_network_interfaces = {}
        network_interfaces = self.__get_network_interfaces()
        for network_interface in network_interfaces:
            for ip_configuration in network_interface.get('ip_configurations', []):
                if ip_configuration.get('public_ip_address', {}):
                    public_ipv4_address_id = ip_configuration.get('public_ip_address', {}).get('id')
                    public_ipv4_network_interfaces.setdefault(public_ipv4_address_id, []).append(network_interface)
        return public_ipv4_network_interfaces

    def describe_nat_gateways(self):
        """
        This method lists all the azure nat gateways
        :return:
        :rtype:
        """
        try:
            nat_gateway_iter_object = self.__network_client.nat_gateways.list_all()
            nat_gateways = self._item_paged_iterator(item_paged_object=nat_gateway_iter_object, as_dict=True)
            return nat_gateways
        except HttpResponseError as http_err:
            logger.error(http_err)
            raise http_err
        except Exception as err:
            raise err

    # delete operations

    def release_public_ip(self, resource_id: str):
        """
        This method releases the public ip
        :return:
        :rtype:
        """
        id_key_pairs = self.get_id_dict_data(resource_id)
        resource_group_name = id_key_pairs.get('resourcegroups')
        public_ip_address_name = id_key_pairs.get('publicipaddresses')
        status = self.__network_client.public_ip_addresses.begin_delete(resource_group_name=resource_group_name,
                                                                        public_ip_address_name=public_ip_address_name)
        return status.done()

    def delete_nat_gateway(self, resource_id: str):
        """
        This method deletes the NatGateway
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        id_key_pairs = self.get_id_dict_data(resource_id)
        resource_group_name = id_key_pairs.get('resourcegroups')
        nat_gateway_name = id_key_pairs.get('natgateways')
        status = self.__network_client.nat_gateways.begin_delete(resource_group_name=resource_group_name,
                                                                 nat_gateway_name=nat_gateway_name)
        return status.done()



import csv

import boto3

from cloud_governance.common.clouds.aws.utils.utils import Utils
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.policy.aws.zombie_cluster_resource import ZombieClusterResources


class ValidateZombies:

    def __init__(self, file_path: str, region: str = 'us-east-2'):
        self.ec2_client = boto3.client('ec2', region_name=region)
        self.cluster_prefix = 'kubernetes.io/cluster/'
        self.__get_details_resource_list = Utils().get_details_resource_list
        self.file_path = file_path
        self.all_instances = ZombieClusterResources(region=region, cluster_prefix=self.cluster_prefix).all_cluster_instance

    def read_csv(self):
        with open(self.file_path) as file:
            csvreader = csv.reader(file)
            rows = []
            for row in csvreader:
                rows.extend(row)
            values = self.all_instances().values()
            logger.info(f'{len(rows)} clusters are validating to be zombie or not')
            for row in rows:
                if row in values:
                    logger.info(f'{row} this is live cluster dont delete it')



import json
from datetime import datetime, timedelta

from azure.core.exceptions import HttpResponseError
from azure.mgmt.monitor import MonitorManagementClient

from cloud_governance.common.clouds.azure.common.common_operations import CommonOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.utils.configs import LOOK_BACK_DAYS, UNUSED_DAYS


class MonitorManagementOperations(CommonOperations):

    def __init__(self):
        super().__init__()
        self.__monitor_client = MonitorManagementClient(credential=self._default_creds,
                                                        subscription_id=self._subscription_id)

    def __get_end_date(self):
        """
        This method returns the end date
        :return:
        :rtype:
        """
        return datetime.utcnow()

    def __get_start_date(self):
        """
        This method returns the start date
        :return:
        :rtype:
        """
        return self.__get_end_date() - timedelta(LOOK_BACK_DAYS)

    def get_audit_records(self, resource_id: str, start_date: datetime = None, end_date: datetime = None):
        """
        This method returns the audit record for the resource_id
        :param start_date:
        :type start_date:
        :param end_date:
        :type end_date:
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        if not start_date:
            start_date = self.__get_start_date()
        if not end_date:
            end_date = self.__get_end_date()
        try:
            filter_query = f"eventTimestamp ge '{start_date}'" \
                           f" and eventTimestamp le '{end_date}' " \
                           f"and resourceUri eq '{resource_id}'.:code:"
            records = self.__monitor_client.activity_logs.list(filter=filter_query)
            return self._item_paged_iterator(item_paged_object=records, as_dict=True)
        except HttpResponseError as http_error:
            logger.error(http_error)
        except Exception as err:
            logger.error(err)

    def get_resource_metrics(self, resource_id: str, metricnames: str, aggregation: str,
                             timespan: str = None, interval: timedelta = timedelta(days=1), **kwargs):
        """
        This method returns the metrics object of individual resource, default timespan will be last 7 days
        :param resource_id:
        :type resource_id:
        :param metricnames:
        :type metricnames:
        :param aggregation:
        :type aggregation:
        :param timespan:
        :type timespan:
        :param interval:
        :type interval:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """
        if not timespan:
            end_date = datetime.utcnow()
            start_date = end_date - timedelta(days=UNUSED_DAYS)
            timespan = f'{start_date}/{end_date}'
        response = self.__monitor_client.metrics.list(resource_uri=resource_id, timespan=timespan,
                                                      metricnames=metricnames, aggregation=aggregation,
                                                      result_type='Data', interval=interval,
                                                      **kwargs)
        return response.as_dict()



from typing import Optional

from azure.core.paging import ItemPaged
from azure.identity import DefaultAzureCredential

from cloud_governance.cloud_resource_orchestration.utils.common_operations import string_equal_ignore_case
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.main.environment_variables import environment_variables


class CommonOperations:

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self._default_creds = DefaultAzureCredential()
        self._subscription_id = self.__environment_variables_dict.get('AZURE_SUBSCRIPTION_ID')

    def _item_paged_iterator(self, item_paged_object: ItemPaged, as_dict: bool = False):
        """
        This method iterates the paged object and return the list
        :param item_paged_object:
        :return:
        """
        iterator_list = []
        try:
            page_item = item_paged_object.next()
            while page_item:
                if as_dict:
                    iterator_list.append(page_item.as_dict())
                else:
                    iterator_list.append(page_item)
                page_item = item_paged_object.next()
        except StopIteration:
            pass
        return iterator_list

    def check_tag_name(self, tags: Optional[dict], tag_name: str, cast_type: str = 'str'):
        """
        This method checks tag is present and return its value
        :param cast_type:
        :type cast_type:
        :param tags:
        :param tag_name:
        :return:
        """
        if tags:
            for key, value in tags.items():
                if string_equal_ignore_case(key, tag_name):
                    return self.__convert_cast_type(value=str(value), type_cast=cast_type)
        return ''

    def __convert_cast_type(self, type_cast: str, value: str):
        """
        This method returns the type conversion value
        :param type_cast:
        :type type_cast:
        :param value:
        :type value:
        :return:
        :rtype:
        """
        if type_cast == 'str':
            return str(value)
        elif type_cast == 'int':
            return int(value)
        elif type_cast == 'float':
            return float(value)
        else:
            return str(value)

    def get_resource_group_name_from_resource_id(self, resource_id: str):
        """
        This method returns the resource_group from resource_id
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        id_list = resource_id.split('/')
        key_values = {id_list[i].lower(): id_list[i+1] for i in range(0, len(id_list) - 1)}
        return key_values.get('resourcegroups').lower()

    def get_id_dict_data(self, resource_id: str):
        """
        This method generates the vm id dictionary
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        pairs = resource_id.split('/')[1:]
        key_pairs = {pairs[i].lower(): pairs[i + 1] for i in range(0, len(pairs), 2)}
        return key_pairs



import os
from datetime import datetime, timedelta
import time
import pandas as pd
from elasticsearch.helpers import bulk

from cloud_governance.main.environment_variables import environment_variables

from elasticsearch_dsl import Search
from elasticsearch import Elasticsearch
from typeguard import typechecked

from cloud_governance.common.elasticsearch.elasticsearch_exceptions import ElasticSearchDataNotUploaded
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp, logger


class ElasticSearchOperations:
    """
    This class related to ElasticSearch operations
    """

    # sleep time between checks is 10 sec
    SLEEP_TIME = 10
    # ElasticSearch fetch data of last 15 minutes
    ES_FETCH_MIN_TIME = 15
    # max search results
    MAX_SEARCH_RESULTS = 1000
    MIN_SEARCH_RESULTS = 100
    DEFAULT_ES_BULK_LIMIT = 5000

    def __init__(self, es_host: str = None, es_port: str = None, region: str = '', bucket: str = '', logs_bucket_key: str = '',
                 timeout: int = 2000):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__es_host = es_host if es_host else self.__environment_variables_dict.get('es_host')
        self.__es_port = es_port if es_port else self.__environment_variables_dict.get('es_port')
        self.__region = region
        self.__timeout = int(self.__environment_variables_dict.get('ES_TIMEOUT')) if self.__environment_variables_dict.get('ES_TIMEOUT') else timeout
        self.__account = self.__environment_variables_dict.get('account')
        try:
            self.__es = Elasticsearch([{'host': self.__es_host, 'port': self.__es_port}], timeout=self.__timeout, max_retries=2)
        except Exception as err:
            self.__es = None

    def __elasticsearch_get_index_hits(self, index: str, uuid: str = '', workload: str = '', fast_check: bool = False,
                                       id: bool = False):
        """
        This method search for data per index in last 2 minutes and return the number of docs or zero
        :param index:
        :param workload: need only if there is different timestamp parameter in Elasticsearch
        :param id: True to return the doc ids
        :param fast_check: return fast response
        :return:
        """
        """
        :return:
        """
        ids = []
        # https://github.com/elastic/elasticsearch-dsl-py/issues/49
        self.__es.indices.refresh(index=index)
        # timestamp name in Elasticsearch is different
        search = Search(using=self.__es, index=index).filter('range', timestamp={
            'gte': f'now-{self.ES_FETCH_MIN_TIME}m', 'lt': 'now'})
        # reduce the search result
        if fast_check:
            search = search[0:self.MIN_SEARCH_RESULTS]
        else:
            search = search[0:self.MAX_SEARCH_RESULTS]
        search_response = search.execute()
        if search_response.hits:
            if uuid:
                count_hits = 0
                for row in search_response:
                    if type(row['uuid']) == str:
                        # uperf return str
                        current_uuid = row['uuid']
                    else:
                        current_uuid = row['uuid'][0]
                    if current_uuid == uuid:
                        if fast_check:
                            return 1
                        ids.append(row.meta.id)
                        count_hits += 1
                if id:
                    return ids
                else:
                    return count_hits
            else:
                return len(search_response.hits)
        else:
            return 0

    @typechecked()
    @logger_time_stamp
    def verify_elasticsearch_data_uploaded(self, index: str, uuid: str = '', workload: str = '',
                                           fast_check: bool = False):
        """
        The method wait till data upload to elastic search and wait if there is new data, search in last 15 minutes
        :param index:
        :param uuid: the current workload uuid
        :param workload: workload name only if there is a different timestamp parameter name in elasticsearch
        :param fast_check: return response on first doc
        :return:
        """
        current_wait_time = 0
        current_hits = 0
        # waiting for any hits
        while current_wait_time <= self.__timeout:
            # waiting for new hits
            new_hits = self.__elasticsearch_get_index_hits(index=index, uuid=uuid, workload=workload,
                                                           fast_check=fast_check)
            if current_hits < new_hits:
                logger.info(f'Data with index: {index} and uuid={uuid} was uploaded to ElasticSearch successfully')
                return self.__elasticsearch_get_index_hits(index=index, uuid=uuid, workload=workload, id=True,
                                                           fast_check=fast_check)
            # sleep for x seconds
            time.sleep(self.SLEEP_TIME)
            current_wait_time += self.SLEEP_TIME
        raise ElasticSearchDataNotUploaded

    @typechecked()
    def upload_to_elasticsearch(self, index: str, data: dict, doc_type: str = '_doc', es_add_items: dict = None,
                                **kwargs):
        """
        This method is upload json data into elasticsearch
        :param index: index name to be stored in elasticsearch
        :param data: data must be in dictionary i.e. {'key': 'value'}
        :param doc_type:
        :param es_add_items:
        :return:
        """
        # read json to dict
        json_path = ""

        # Add items
        if es_add_items:
            for key, value in es_add_items.items():
                data[key] = value

        # utcnow - solve timestamp issue
        if not data.get('timestamp'):
            data['timestamp'] = datetime.utcnow()  # datetime.now()
        if 'policy' not in data:
            data['policy'] = self.__environment_variables_dict.get('policy')
        # Upload data to elastic search server
        if 'account' not in map(str.lower, data.keys()):
            data['account'] = self.__account
        if data.get('index-id'):
            kwargs['id'] = data.get('index-id')
        if 'DryRun' not in data:
            data['DryRun'] = self.__environment_variables_dict.get('dry_run')
        if 'CleanUpDays' not in data:
            data['CleanUpDays'] = self.__environment_variables_dict.get('DAYS_TO_TAKE_ACTION')
        try:
            if isinstance(data, dict):  # JSON Object
                self.__es.index(index=index, doc_type=doc_type, body=data, **kwargs)
            else:  # JSON Array
                for record in data:
                    self.__es.index(index=index, doc_type=doc_type, body=record, **kwargs)
            return True
        except Exception as err:
            raise err

    @typechecked()
    def update_elasticsearch_index(self, index: str, id: str, metadata: dict = ''):
        """
        This method update existing index
        :param index: index name
        :param id: The specific index id
        :param metadata: The metadata for enrich that existing index according to id
        :return:
        """
        self.__es.update(index=index, id=id, body={"doc": metadata})

    @typechecked()
    @logger_time_stamp
    def get_elasticsearch_index_by_id(self, index: str, id: str):
        """
        This method return elastic search index data by id
        :param index: index name
        :param id: The specific index id
        :return:
        """
        return self.__es.get(index=index, id=id)

    @typechecked()
    @logger_time_stamp
    def get_index_hits(self, days: int, index: str):
        """
        This method return the last days data from elastic search
        @param days:
        @param index:
        @return:
        """
        search = Search(using=self.__es, index=index).filter('range', timestamp={'gte': f'now-{days}d', 'lt': 'now'})
        search = search[0:self.MAX_SEARCH_RESULTS]
        search_response = search.execute()
        df = pd.DataFrame()
        for row in search_response:
            df = pd.concat([df, pd.DataFrame([row.to_dict()])], ignore_index=True).fillna({})
        return df.to_dict('records')

    @typechecked()
    @logger_time_stamp
    def get_query_data_between_range(self, start_datetime: datetime, end_datetime: datetime):
        """
        This method returns the query to fetch the data in between ranges
        @return:
        """
        query = {
            "query": {
                "bool": {
                    "filter": {
                        "range": {
                            "timestamp": {
                                "format": "yyyy-MM-dd HH:mm:ss"
                            }
                        }
                    }
                }
            }
        }
        query['query']['bool']['filter']['range']['timestamp']['lte'] = str(end_datetime.replace(microsecond=0))
        query['query']['bool']['filter']['range']['timestamp']['gte'] = str(start_datetime.replace(microsecond=0))
        return query

    @typechecked()
    def fetch_data_by_es_query(self, es_index: str, query: dict = None, start_datetime: datetime = None,
                               end_datetime: datetime = None, result_agg: bool = False, group_by: str = '', search_size: int = 100, limit_to_size: bool = False,
                               filter_path: str = ''):
        """
        This method fetches the data in between range, if you need aggregation results pass you own query with aggegation
        @param es_index:
        @param start_datetime:
        @param end_datetime:
        @param query:
        @param result_agg:
        @param group_by:
        @param search_size:
        @param limit_to_size: limit to size
        @param filter_path:
        @return:
        """
        es_data = []
        if self.__es.indices.exists(index=es_index):
            if not query:
                if start_datetime and end_datetime:
                    query = self.get_query_data_between_range(start_datetime=start_datetime, end_datetime=end_datetime)
            if query:
                response = self.__es.search(index=es_index, body=query, doc_type='_doc', size=search_size, scroll='1h', filter_path=filter_path)
                if result_agg:
                    es_data.extend(response.get('aggregations').get(group_by).get('buckets'))
                else:
                    if response.get('hits', {}).get('hits'):
                        es_data.extend(response.get('hits').get('hits'))
                    if not limit_to_size:
                        scroll_id = response.get('_scroll_id')
                        while scroll_id:
                            response = self.__es.scroll(scroll_id=scroll_id, scroll="1h")
                            if len(response.get('hits').get('hits')) > 0:
                                es_data.extend(response.get('hits').get('hits'))
                            else:
                                break
        return es_data

    @typechecked()
    @logger_time_stamp
    def delete_data_in_between_in_es(self, es_index: str, start_datetime: datetime, end_datetime: datetime):
        """
        This method deletes the data in between two ranges
        @param es_index:
        @param start_datetime:
        @param end_datetime:
        @return:
        """
        if self.__es.indices.exists(index=es_index):
            query_body = self.get_query_data_between_range(start_datetime=start_datetime, end_datetime=end_datetime)
            logger.info(f'Clearing data from {start_datetime} to  {end_datetime} ')
            return self.__es.delete_by_query(index=es_index, body=query_body)

    @typechecked()
    @logger_time_stamp
    def delete_data_in_es(self, es_index: str):
        """
        This method delete the data in the index
        @param es_index: index_name
        @return:
        """
        if self.__es.indices.exists(index=es_index):
            return self.__es.indices.delete(index=es_index)

    def verify_elastic_index_doc_id(self, index: str, doc_id: str):
        """
        This method verify that document present in the index or not
        """
        return self.__es.exists(index=index, id=doc_id)

    def get_es_data_by_id(self, id: str, index: str):
        """This method fetch the data from the es based on the match case and match value"""
        try:
            es_data = self.__es.get(index=index, id=id)
        except Exception as err:
            es_data = {}
        return es_data

    def upload_data_in_bulk(self, data_items: list, index: str, **kwargs):
        """
        This method uploads the data using the bulk api
        :param index:
        :param data_items:
        :return:
        """
        total_uploaded = 0
        failed_items = 0
        for i in range(0, len(data_items), self.DEFAULT_ES_BULK_LIMIT):
            bulk_items = data_items[i: i + self.DEFAULT_ES_BULK_LIMIT]
            for item in bulk_items:
                if kwargs.get('id'):
                    item['_id'] = item.get(kwargs.get('id'))
                if item.get('index-id'):
                    item['_id'] = item.get('index-id')
                if not item.get('timestamp'):
                    if 'CurrentDate' in item:
                        item['timestamp'] = datetime.strptime(item.get('CurrentDate'), "%Y-%m-%d")
                    else:
                        item['timestamp'] = datetime.utcnow()
                item['_index'] = index
                if item.get('AccountId'):
                    item['AccountId'] = str(item.get('AccountId'))
                if 'account' not in item:
                    item['account'] = self.__account
                if 'DryRun' not in item:
                    item['DryRun'] = self.__environment_variables_dict.get('dry_run')
                if 'CleanUpDays' not in item:
                    item['ExpireDays'] = self.__environment_variables_dict.get('DAYS_TO_TAKE_ACTION')
                item['policy'] = self.__environment_variables_dict.get('policy')
            response = bulk(self.__es, bulk_items)
            if response:
                total_uploaded += len(bulk_items)
            else:
                failed_items += len(bulk_items)
        if total_uploaded > 0:
            logger.info(f"✅️ {total_uploaded} is uploaded to the elastic search index: {index}")
        if failed_items > 0:
            logger.error(f"❌ {failed_items} is not uploaded to the elasticsearch index: {index}")

    def check_elastic_search_connection(self):
        """
        This method returns boolean value on elasticsearch connection
        :return:
        """
        if self.__es:
            return self.__es.ping()
        return False

    def post_query(self, query: dict, es_index: str, result_agg: bool = False):
        """
        This method returns the es data
        :param result_agg:
        :type result_agg:
        :param query:
        :type query:
        :param es_index:
        :type es_index:
        :return:
        :rtype:
        """
        try:
            response = self.__es.search(index=es_index, body=query)
            if result_agg:
                return response.get('aggregations')
            else:
                return response.get('hits', {}).get('hits', {})
        except Exception as err:
            logger.error(err)
            raise err



import os

import boto3
import typeguard
from typing import Callable

from cloud_governance.common.clouds.aws.utils.common_methods import get_boto3_client
from cloud_governance.common.clouds.aws.utils.utils import Utils
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables


class EC2Operations:
    """
    This class is useful for writing EC2 Operations
    """

    TAG_BATCHES = 20

    def __init__(self, region: str = 'us-east-2'):
        """
        Initializing the AWS resources
        """
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.elb1_client = get_boto3_client('elb', region_name=region)
        self.elbv2_client = get_boto3_client('elbv2', region_name=region)
        self.ec2_client = get_boto3_client('ec2', region_name=region)
        self.get_full_list = Utils().get_details_resource_list
        self.utils = Utils(region=region)

    @logger_time_stamp
    @typeguard.typechecked
    def find_load_balancer(self, elb_name: str):
        """
        Find the load balancer is present or not
        :param elb_name:
        :return:
        """
        # elbs = self.get_full_list()
        elbs = self.elb1_client.describe_load_balancers()['LoadBalancerDescriptions']
        if elbs:
            for elb in elbs:
                if elb['LoadBalancerName'] == elb_name:
                    return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_load_balancer_v2(self, elb_name: str):
        """
        Find the load balancer is present or not
        :param elb_name:
        :return:
        """
        elbs = self.elbv2_client.describe_load_balancers()['LoadBalancers']
        if elbs:
            for elb in elbs:
                if elb['LoadBalancerName'] == elb_name:
                    return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_vpc_endpoints(self, vpc_endpoint_id: str):
        """
        Find the vpc endpoints present or not
        :param vpc_endpoint_id:
        :return:
        """
        vpc_endpoints = self.ec2_client.describe_vpc_endpoints()['VpcEndpoints']
        if vpc_endpoints:
            for vpc_endpoint in vpc_endpoints:
                if vpc_endpoint['VpcEndpointId'] == vpc_endpoint_id:
                    if vpc_endpoint['State'] == 'deleted':
                        return True
            return False
        return False

    @logger_time_stamp
    def find_volume(self):
        """
        Find the volumes is present or nor
        :return:
        """
        volumes = self.ec2_client.describe_volumes()
        if len(volumes['Volumes']) == 0:
            return True
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_dhcp_options(self, dhcp_id: str):
        """
        Find the DHCP option present or not
        :return:
        """
        dhcp_options = self.ec2_client.describe_dhcp_options()['DhcpOptions']
        if dhcp_options:
            for dhcp_option in dhcp_options:
                if dhcp_option['DhcpOptionsId'] == dhcp_id:
                    return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_snapshots(self, snapshot_id: str):
        """
        Find the snapshots present or not
        :param snapshot_id:
        :return:
        """
        snapshots = self.ec2_client.describe_snapshots()['Snapshots']
        if snapshots:
            for snapshot in snapshots:
                if snapshot['SnapshotId'] == snapshot_id:
                    return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_route_table(self, route_table_id: str):
        """
        Find the route table is present or not
        :param route_table_id:
        :return:
        """
        route_tables = self.ec2_client.describe_route_tables()['RouteTables']
        if route_tables:
            for route_table in route_tables:
                if route_table['RouteTableId'] == route_table_id:
                    return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_security_group(self, security_group_id: str):
        """
        Find the security group is present or not
        :param security_group_id:
        :return:
        """
        security_groups = self.ec2_client.describe_security_groups()['SecurityGroups']
        if security_groups:
            for security_group in security_groups:
                if security_group['GroupId'] == security_group_id:
                    return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_nat_gateway(self, nat_gateway_id: str):
        """
        find the nat gateway is present or not
        :param nat_gateway_id:
        :return:
        """
        nat_gateways = self.ec2_client.describe_nat_gateways()['NatGateways']
        if nat_gateways:
            for nat_gateway in nat_gateways:
                if nat_gateway['NatGatewayId'] == nat_gateway_id:
                    if nat_gateway['State'] == 'deleted':
                        return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_network_acl(self, network_acl_id: str):
        """
        Find the network acl is present or not
        :param network_acl_id:
        :return:
        """
        network_acls = self.ec2_client.describe_network_acls()['NetworkAcls']
        if network_acls:
            for network_acl in network_acls:
                if network_acl['NetworkAclId'] == network_acl_id:
                    return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_network_interface(self, network_interface_id: str):
        """
        find the network interface is present or not
        :param network_interface_id:
        :return:
        """
        network_interfaces = self.ec2_client.describe_network_interfaces()['NetworkInterfaces']
        if network_interfaces:
            for network_interface in network_interfaces:
                if network_interface['NetworkInterfaceId'] == network_interface_id:
                    return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_internet_gateway(self, ing_id: str):
        """
        find the internet gateway is presnt or not
        :param ing_id:
        :return:
        """
        internet_gateways = self.ec2_client.describe_network_interfaces()['NetworkInterfaces']
        if internet_gateways:
            for internet_gateway in internet_gateways:
                if internet_gateway['InternetGatewayId'] == ing_id:
                    return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_subnet(self, subnet_id: str):
        """
        find the subnet is present or not
        :param subnet_id:
        :return:
        """
        subnets = self.ec2_client.describe_subnets()['Subnets']
        if subnets:
            for subnet in subnets:
                if subnet['SubnetId'] == subnet_id:
                    return True
            return False
        return False

    @logger_time_stamp
    def find_elastic_ip(self):
        """
        find the elastic ip is present or not
        :return:
        """
        elastic_ips = self.ec2_client.describe_addresses()['Addresses']
        if len(elastic_ips) == 0:
            return True
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_vpc(self, cluster_tag: str):
        """
        find the vpc present or not
        :param cluster_tag:
        :return:
        """
        vpcs = self.ec2_client.describe_vpcs()['Vpcs']
        if vpcs:
            for vpc in vpcs:
                if vpc.get('tags'):
                    for tag in vpc.get('Tags'):
                        if tag['Key'] == cluster_tag:
                            return True
            return False
        return False

    @logger_time_stamp
    @typeguard.typechecked
    def find_ami(self, image_id: str):
        """
        find the amazon machine image is present or not
        :param image_id:
        :return:
        """
        images = self.ec2_client.describe_images(Owners=['self'])
        if images:
            for image in images['Images']:
                if image['ImageId'] == image_id:
                    return True
            return False
        return False

    @typeguard.typechecked
    def scan_cluster_or_non_cluster_instance(self, resources: list):
        """
        This method returns the list of cluster and non-cluster instances.
        @param resources:
        @return:
        """
        cluster = []
        non_cluster = []
        for resource in resources:
            found = False
            for item in resource:
                if item.get('Tags'):
                    for tag in item.get('Tags'):
                        if 'kubernetes.io/cluster/' in tag.get('Key'):
                            found = True
                            break
            if found:
                cluster.append(resource)
            else:
                non_cluster.append(resource)
        return [cluster, non_cluster]

    @typeguard.typechecked
    def scan_cluster_non_cluster_resources(self, resources: list, tags: str = 'Tags'):
        """
        This method returns the list of cluster and non-cluster resources.
        @param tags:
        @param resources:
        @return:
        """
        cluster = []
        non_cluster = []
        for resource in resources:
            found = False
            if resource.get(tags):
                for tag in resource.get(tags):
                    if 'kubernetes.io/cluster/' in tag.get('Key'):
                        found = True
                        break
            if found:
                cluster.append(resource)
            else:
                non_cluster.append(resource)
        return [cluster, non_cluster]

    def get_instances(self, ec2_client: boto3.client = None, **kwargs):
        """
        This method returns all instances from the region
        @return:
        """
        describe_instances = ec2_client.describe_instances if ec2_client else self.ec2_client.describe_instances
        return self.utils.get_details_resource_list(func_name=describe_instances,
                                                    input_tag='Reservations', check_tag='NextToken', **kwargs)

    def get_volumes(self, **kwargs):
        """
        This method returns all volumes in the region
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_volumes, input_tag='Volumes',
                                                    check_tag='NextToken', **kwargs)

    def get_images(self):
        """
        This method returns all images in the region
        @return:
        """
        return self.ec2_client.describe_images(Owners=['self'])['Images']

    def get_snapshots(self):
        """
        This method returns all snapshots in the region
        @return:
        """
        return self.ec2_client.describe_snapshots(OwnerIds=['self'])['Snapshots']

    def get_security_groups(self):
        """
        This method returns security groups in the region
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_security_groups,
                                                    input_tag='SecurityGroups', check_tag='NextToken')

    def get_elastic_ips(self):
        """
        This method returns elastic_ips in the region
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_addresses, input_tag='Addresses',
                                                    check_tag='NextToken')

    def get_network_interface(self):
        """
        This method returns network_interface in the region
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_network_interfaces,
                                                    input_tag='NetworkInterfaces', check_tag='NextToken')

    def get_load_balancers(self):
        """
        This method returns load balancers in the region
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.elb1_client.describe_load_balancers,
                                                    input_tag='LoadBalancerDescriptions', check_tag='Marker')

    def get_load_balancers_v2(self):
        """
        This method returns load balancers v2 in the region
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.elbv2_client.describe_load_balancers,
                                                    input_tag='LoadBalancers', check_tag='Marker')

    def get_vpcs(self):
        """
        This method returns all vpcs
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_vpcs, input_tag='Vpcs',
                                                    check_tag='NextToken')

    def get_subnets(self):
        """
        This method returns all subnets
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_subnets, input_tag='Subnets',
                                                    check_tag='NextToken')

    def get_route_tables(self):
        """
        This method returns all route tables
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_route_tables,
                                                    input_tag='RouteTables', check_tag='NextToken')

    def get_internet_gateways(self):
        """
        This method returns all internet gateways
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_internet_gateways,
                                                    input_tag='InternetGateways', check_tag='NextToken')

    def get_dhcp_options(self):
        """
        This method returns all dhcp options
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_dhcp_options,
                                                    input_tag='DhcpOptions', check_tag='NextToken')

    def get_vpce(self):
        """
        This method returns all vpc endpoints
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_vpc_endpoints,
                                                    input_tag='VpcEndpoints', check_tag='NextToken')

    def get_nat_gateways(self):
        """
        This method returns all nat gateways
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_nat_gateways,
                                                    input_tag='NatGateways', check_tag='NextToken')

    def get_nacls(self):
        """
        This method returns all network acls
        @return:
        """
        return self.utils.get_details_resource_list(func_name=self.ec2_client.describe_network_acls,
                                                    input_tag='NetworkAcls', check_tag='NextToken')

    def is_cluster_resource(self, resource_id: str):
        """
        This method checks tags have cluster key, if cluster return True else False
        @param resource_id:
        @return:
        """
        resource_tags = self.ec2_client.describe_tags(Filters=[{'Name': 'resource-id', 'Values': [resource_id]}])
        if resource_tags.get('Tags'):
            for tag in resource_tags['Tags']:
                if 'kubernetes.io/cluster/' in tag.get('Key'):
                    return True
        return False

    def get_ec2_list(self, instances_list: list):
        """
        This method return all instances in one list by taking instances_list
        @param instances_list:
        @return:
        """
        instances = []
        for resources in instances_list:
            for resource in resources['Instances']:
                instances.append(resource)
        return instances

    def get_tag(self, name: str, tags: list):
        """
        This method get tag name fom the tags
        @param name:
        @param tags:
        @return:
        """
        if tags:
            for tag in tags:
                if tag.get('Key') == name:
                    return tag.get('Value')
        return 'NA'

    def get_global_ec2_list_by_user(self):
        """
        This method get ec2-instances based on User tag
        @return:
        """
        users_list = {}
        regions = self.ec2_client.describe_regions()['Regions']
        for region in regions:
            region_ec2_client = boto3.client('ec2', region_name=region.get('RegionName'))
            instances = self.get_ec2_list(region_ec2_client.describe_instances()['Reservations'])
            for instance in instances:
                user_data = {'InstanceId': instance.get('InstanceId'),
                             'Name': self.get_tag(name='Name', tags=instance.get('Tags')),
                             'InstanceType': instance.get('InstanceType'),
                             'LaunchTime': instance.get('LaunchTime').strftime('%Y/%m/%d %H:%M:%S'),
                             'Region': region.get('RegionName'),
                             'Account': self.__environment_variables_dict.get('account', '').upper(),
                             'State': instance.get('State')['Name']
                             }
                user = self.get_tag(name='User', tags=instance.get('Tags'))
                if user in users_list:
                    users_list[user].append(user_data)
                else:
                    users_list[user] = [user_data]
        return users_list

    def get_tag_value_from_tags(self, tags: list, tag_name: str, cast_type: str = 'str',
                                default_value: any = '') -> any:
        """
        This method return the tag value inputted by tag_name
        """
        if tags:
            for tag in tags:
                key = tag.get('Key').lower().replace("_", '').replace("-", '').strip()
                if key == tag_name.lower():
                    if cast_type:
                        if cast_type == 'int':
                            return int(tag.get('Value').split()[0].strip())
                        elif cast_type == 'float':
                            return float(tag.get('Value').strip())
                        else:
                            return str(tag.get('Value').strip())
                    return tag.get('Value').strip()
        return default_value

    def get_active_regions(self):
        """
        This method return active regions in aws account
        :return:
        """
        responses = self.ec2_client.describe_regions()['Regions']
        active_regions = []
        for region in responses:
            active_regions.append(region.get('RegionName'))
        return active_regions

    def get_ec2_instance_list(self, **kwargs):
        """
        This method returns the list of instances
        :param kwargs:
        :return:
        """
        instances_list = []
        ignore_tag = kwargs.pop('ignore_tag', None)
        instances = self.get_instances(**kwargs)
        for instance in instances:
            for resource in instance['Instances']:
                skip_resource = False
                if ignore_tag:
                    for tag in resource.get('Tags', []):
                        if tag.get('Key') == ignore_tag:
                            skip_resource = True
                            break
                if not skip_resource:
                    instances_list.append(resource)
        return instances_list

    def get_ec2_instance_ids(self, **kwargs):
        """
        This method return the ec2 instance ids
        :param kwargs:
        :return:
        """
        instances = self.get_ec2_instance_list(**kwargs)
        instance_ids = []
        for instance in instances:
            instance_ids.append(instance.get('InstanceId'))
        return instance_ids

    def tag_ec2_resources(self, client_method: Callable, tags: list, resource_ids: list):
        """
        This method tag the ec2 resources with batch wise of 10
        :param client_method:
        :param tags:
        :param resource_ids:
        :return:
        """
        co = 0
        for start in range(0, len(resource_ids), self.TAG_BATCHES):
            end = start + self.TAG_BATCHES
            client_method(Resources=resource_ids[start:end], Tags=tags)
            co += 1
        return co

    def get_attached_time(self, volume_list: list):
        """
        This method return the root volume attached time
        :param volume_list:
        :return:
        """
        for mapping in volume_list:
            if mapping.get('Ebs').get('DeleteOnTermination'):
                return mapping.get('Ebs').get('AttachTime')
        return ''

    def get_active_instances(self, tag_name: str, tag_value: str, skip_full_scan: bool = False, ignore_tag: str = ''):
        """
        This method returns all active instances by filter tag_name, tag_value in all active regions
        :param ignore_tag:
        :param skip_full_scan:
        :param tag_name:
        :param tag_value:
        :return:
        """
        active_instances = {}
        active_regions = self.get_active_regions()
        for region_name in active_regions[::-1]:
            filters = [{'Name': f'tag:{tag_name}', 'Values': [tag_value, tag_value.upper(), tag_value.lower(), tag_value.title()]}]
            self.get_ec2_instance_list()
            active_instances_in_region = self.get_ec2_instance_list(Filters=filters, ec2_client=boto3.client('ec2', region_name=region_name), ignore_tag=ignore_tag)
            if active_instances_in_region:
                if skip_full_scan:
                    return True
                else:
                    active_instances[region_name] = active_instances_in_region
        return False if skip_full_scan else active_instances

    def verify_active_instances(self, tag_name: str, tag_value: str):
        """
        This method verify any active instances in all regions by tag_name, tag_value
        :param tag_name:
        :param tag_value:
        :return:
        """
        ignore_tag = 'TicketId'
        return self.get_active_instances(tag_name=tag_name, tag_value=tag_value, skip_full_scan=True, ignore_tag=ignore_tag)

    def describe_tags(self, **kwargs):
        """
        This method returns the all tags in aws region
        :param kwargs:
        :return:
        """
        tags_list = []
        ec2_service_tags = self.ec2_client.describe_tags(**kwargs)
        tags_list.extend(ec2_service_tags.get('Tags', []))
        while ec2_service_tags.get('NextToken'):
            ec2_service_tags = self.ec2_client.describe_tags(NextToken=ec2_service_tags.get('NextToken'), **kwargs)
            tags_list.extend(ec2_service_tags.get('Tags', []))
        return tags_list

    def get_running_instance(self):
        """
        This method returns the EC2 running instances
        :return:
        :rtype:
        """
        return self.get_ec2_instance_list(Filters=[{'Name': 'instance-state-name', 'Values': ['running']}])



from typing import Union

from cloud_governance.main.environment_variables import environment_variables


def string_equal_ignore_case(value1: str, value2: str, *args) -> bool:
    """
    This method finds all values are equal and returns bool
    :return:
    """
    equal = value1.lower() == value2.lower()
    if args:
        for val in args:
            equal = value1.lower() == val.lower() and equal
    return equal


def integer_equal(value1: int, value2: int, *args) -> bool:
    """
    This method finds all values are equal and returns bool
    :param value1:
    :param value2:
    :param args:
    :return:
    """
    equal = value1 == value2
    if args:
        for val in args:
            equal = equal and value1 == val
    return equal


def get_tag_value_by_name(tags: list, tag_name: str) -> str:
    """
    This method returns the tag_value
    :param tags:
    :param tag_name:
    :return:
    """
    for tag in tags:
        key = tag.get('Key')
        value = tag.get('Value')
        if string_equal_ignore_case(key, tag_name):
            return value
    return ''


def get_ldap_user_data(user: str, tag_name: str):
    """
    This method returns the ldap user tag_name
    :param user:
    :param tag_name:
    :return:
    """
    from cloud_governance.common.ldap.ldap_search import LdapSearch
    ldap_search = LdapSearch(ldap_host_name=environment_variables.environment_variables_dict.get('LDAP_HOST_NAME', ''))
    user_details = ldap_search.get_user_details(user)
    if user_details:
        return user_details.get(tag_name)
    return 'NA'


def check_name_and_get_key_from_tags(tags: Union[list, dict], tag_name: str, default: str = '', replace_spl: bool = False) -> [str, str]:
    """
    This method returns the key and value if tag_name present in the key
    :param replace_spl:
    :type replace_spl:
    :param default:
    :type default:
    :param tags:
    :type tags:
    :param tag_name:
    :type tag_name:
    :return:
    :rtype:
    """
    if tags:
        if type(tags) == list:
            tags = {tag.get('Key'): tag.get('Value') for tag in tags}
        for key, value in tags.items():
            if replace_spl:
                key = key.lower().replace("_", '').replace("-", '').strip()
            else:
                key = key.lower().strip()
            if tag_name.lower() in key:
                return key, value
    return default, default



import datetime
import smtplib
from email.message import EmailMessage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from typing import Union, Optional

from cloud_governance.common.clouds.aws.s3.s3_operations import S3Operations
from cloud_governance.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
from cloud_governance.common.ldap.ldap_search import LdapSearch
from cloud_governance.common.logger.init_logger import logger

# https://github.com/redhat-performance/quads/blob/master/quads/tools/postman.py
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables


class Postfix:
    """
    This class send mail using the postfix configuration
    install postfix and configure on machine
    $ dnf install postfix -y
    $ vi /etc/postfix/main.cf
    relayhost = [smtp.corp.redhat.com]
    inet_interfaces = all
    inet_protocols = all
    relay_domains = rdu2.scalelab.redhat.com, redhat.com
    $ systemctl restart postfix
    """

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__LDAP_HOST_NAME = self.__environment_variables_dict.get('LDAP_HOST_NAME')
        self.__ldap_search = LdapSearch(ldap_host_name=self.__LDAP_HOST_NAME)
        self.reply_to = self.__environment_variables_dict.get('REPLY_TO', 'dev-null@redhat.com')
        self.__es_host = self.__environment_variables_dict.get('es_host', '')
        self.__policy = self.__environment_variables_dict.get('policy', '')
        self.__es_port = self.__environment_variables_dict.get('es_port', '')
        self.__account = self.__environment_variables_dict.get('account', '')
        self.__policy_output = self.__environment_variables_dict.get('policy_output', '')
        self.__default_admins = self.__environment_variables_dict.get('DEFAULT_ADMINS')
        self.__email_alert = self.__environment_variables_dict.get('EMAIL_ALERT')
        self.__mail_to = self.__environment_variables_dict.get('EMAIL_TO')
        self.__mail_cc = self.__environment_variables_dict.get('EMAIL_CC')
        self.bucket_name, self.key = self.get_bucket_name()
        self.__es_index = 'cloud-governance-mail-messages'
        if self.__es_host:
            self.__es_operations = ElasticSearchOperations(es_host=self.__es_host, es_port=self.__es_port)
        if self.__policy_output:
            self.__s3_operations = S3Operations(region_name='us-east-1')

    def get_bucket_name(self):
        key = 'logs'
        if 's3' in self.__policy_output.lower():
            targets = self.__policy_output.split('/')
            bucket_name = targets[2]
            key = targets[3]
        else:
            bucket_name = self.__policy_output
        return bucket_name, key

    def split_emails(self, email_str: str):
        """
        This method split the mail list
        :param email_str:
        :type email_str:
        :return:
        :rtype:
        """
        splitter = "," if "," in email_str else " "
        return [item.strip() for item in email_str.split(splitter)]

    def prettify_to(self, to: Union[str, list]):
        """
        This method prettify to
        :param to:
        :type to:
        :return:
        :rtype:
        """

        if isinstance(to, str):
            to = self.split_emails(email_str=to)
        if isinstance(to, list):
            return ','.join([item if '@redhat.com' in item else f'{item}@redhat.com' for item in to])
        return to

    def prettify_cc(self, cc: Union[str, list], to: str = ''):
        """
        This method prettify cc
        :param to:
        :type to:
        :param cc:
        :type cc:
        :return:
        :rtype:
        """
        if isinstance(cc, str):
            cc = self.split_emails(email_str=cc)
        cc_unique_values = [cc_user if '@redhat.com' in cc_user else f'{cc_user}@redhat.com' for cc_user in cc]
        to = self.prettify_to(to=to).split(',')
        return ','.join(list(set(cc_unique_values) - set(to)))

    @logger_time_stamp
    def send_email_postfix(self, subject: str, to: any, cc: list, content: str, **kwargs):
        if self.__email_alert:
            if self.__mail_to:
                to = self.__mail_to
            if self.__mail_cc:
                cc = self.__mail_cc
            if not self.__ldap_search.get_user_details(user_name=to):
                cc.append('athiruma@redhat.com')
            msg = MIMEMultipart('alternative')
            msg["Subject"] = subject
            msg["From"] = "%s <%s>" % (
                'cloud-governance',
                "@".join(["noreply-cloud-governance", 'redhat.com']),
            )
            msg["To"] = self.prettify_to(to)
            msg["Cc"] = self.prettify_cc(cc=cc, to=to)
            # msg.add_header("Reply-To", self.reply_to)
            # msg.add_header("User-Agent", self.reply_to)
            if kwargs.get('filename'):
                attachment = MIMEText(open(kwargs['filename']).read())
                attachment.add_header('Content-Disposition', 'attachment',
                                      filename=kwargs['filename'].split('/')[-1])
                msg.attach(attachment)
            if kwargs.get('mime_type'):
                msg.attach(MIMEText(content, kwargs.get('mime_type')))
            else:
                msg.attach(MIMEText(content))
            email_string = msg.as_string()
            email_host = 'localhost'
            try:
                with smtplib.SMTP(email_host) as s:
                    try:
                        logger.debug(email_string)
                        s.send_message(msg)
                        if isinstance(to, str):
                            logger.warn(f'Mail sent successfully to {to}@redhat.com')
                        elif isinstance(to, list):
                            logger.warn(f'Mail sent successfully to {", ".join(to)}@redhat.com')
                        if kwargs.get('filename'):
                            file_name = kwargs['filename'].split('/')[-1]
                            date_key = datetime.datetime.now().strftime("%Y%m%d%H")
                            if self.__policy_output:
                                self.__s3_operations.upload_file(file_name_path=kwargs['filename'],
                                                                 bucket=self.bucket_name, key=f'{self.key}/{self.__policy}/{date_key}',
                                                                 upload_file=file_name)
                                s3_path = f'{self.__policy_output}/logs/{self.__policy}/{date_key}/{file_name}'
                                content += f'\n\nresource_file_path: s3://{s3_path}\n\n'
                        es_data = kwargs.get('es_data')
                        data = {'Policy': self.__policy, 'To': to, 'Cc': cc, 'Message': content, 'Account': self.__account.upper(), 'MessageType': kwargs.get('message_type', 'alert')}
                        if es_data:
                            data.update(es_data)
                        if kwargs.get('resource_id'):
                            data['resource_id'] = kwargs['resource_id']
                        if kwargs.get('extra_purse'):
                            data['extra_purse'] = round(kwargs['extra_purse'], 3)
                        if self.__es_host:
                            # self.__es_operations.upload_to_elasticsearch(data=data, index=self.__es_index)
                            logger.warn(f'Uploaded to es index: {self.__es_index}')
                        else:
                            logger.warn('Error missing the es_host')
                    except smtplib.SMTPException as ex:
                        logger.error(f'Error while sending mail, {ex}')
                        return False
                return True
            except Exception as err:
                logger.error(f'Some error occurred, {err}')
                return False



from cloud_governance.cloud_resource_orchestration.clouds.azure.resource_groups.abstract_resource import \
    AbstractResource
from cloud_governance.cloud_resource_orchestration.utils.common_operations import get_ldap_user_data
from cloud_governance.common.jira.jira_operations import JiraOperations
from cloud_governance.common.logger.init_logger import logger


class TagCROResources(AbstractResource):
    """
    This class manages the tagging resources which have the tag TicketId
    """

    TICKET_ID = 'TicketId'
    DURATION = 'Duration'
    IN_PROGRESS = 'INPROGRESS'

    def __init__(self):
        super().__init__()
        self._resource_groups = self._resource_group_operations.get_all_resource_groups()

    def __tag_ticket_id_found_resources(self, resource_ids: list, ticket_id: str):
        """
        This method tags the resources with ticket_id data
        :param resource_ids:
        :param ticket_id:
        :return:
        """
        jira_operations = JiraOperations()
        ticket_id = ticket_id.split('-')[-1]
        ticket_description = jira_operations.get_issue_description(ticket_id=ticket_id, state=self.IN_PROGRESS)
        if ticket_description:
            duration = int(ticket_description.get('Days', 0))
            extended_duration = int(jira_operations.get_issue_sub_tasks_duration(ticket_id=ticket_id))
            duration += extended_duration
            estimated_cost = float(ticket_description.get('CostEstimation'))
            budget_extend_ticket_ids = jira_operations.get_budget_extend_tickets(ticket_id=ticket_id,
                                                                                 ticket_state='closed')
            extended_budget = jira_operations.get_total_extend_budget(sub_ticket_ids=budget_extend_ticket_ids)
            estimated_cost = int(estimated_cost) + int(extended_budget)
            manager_approved = ticket_description.get('ApprovedManager')
            user_email = ticket_description.get('EmailAddress')
            user = user_email.split('@')[0]
            project = ticket_description.get('Project')
            adding_extra_tags = {'Duration': str(duration), 'EstimatedCost': str(estimated_cost),
                                 'ApprovedManager': manager_approved, 'Project': project,
                                 'Email': user_email, 'UserCRO': user,
                                 'Manager': get_ldap_user_data(user=user, tag_name='ManagerName').upper(),
                                 'Owner': get_ldap_user_data(user=user, tag_name="FullName").upper(),
                                 'TicketId': ticket_id
                                 }
            tagged_resource_ids = []
            for resource_id in resource_ids:
                success = self._resource_group_operations.creates_or_updates_tags(resource_id=resource_id, tags=adding_extra_tags)
                if success:
                    tagged_resource_ids.append(resource_id)
            logger.info(f"Tagged the resources: {tagged_resource_ids}")

    def __tag_instances(self):
        """
        This method list the instances and tag the instances which have the tag TicketId
        :return:
        """
        ticket_id_instances = {}
        for resource_group in self._resource_groups:
            name = resource_group.name
            resource_group_tags = resource_group.tags
            found_tag_value = self._resource_group_operations.check_tag_name(tags=resource_group_tags, tag_name=self.TICKET_ID)
            found_duration = self._resource_group_operations.check_tag_name(tags=resource_group_tags, tag_name=self.DURATION)
            resources_list = self._resource_group_operations.get_all_resources(resource_group_name=name)
            apply_tags = False
            if found_tag_value and not found_duration:
                apply_tags = True
            if not found_tag_value:
                for resource in resources_list:
                    resource_tags = resource.tags
                    found_tag_value = self._resource_group_operations.check_tag_name(tags=resource_tags, tag_name=self.TICKET_ID)
                    if found_tag_value:
                        if not found_duration:
                            found_duration = self._resource_group_operations.check_tag_name(tags=resource_tags,  tag_name=self.DURATION)
                        if not found_duration:
                            apply_tags = True
                            break
            if apply_tags:
                resource_ids = [resource_group.id]
                for resource in resources_list:
                    resource_ids.append(resource.id)
                self.__tag_ticket_id_found_resources(resource_ids=resource_ids, ticket_id=found_tag_value)
                ticket_id_instances.setdefault(found_tag_value, []).append(resource_ids)

    def run(self):
        """=-
        This method run the tag instance methods
        :return:
        """
        return self.__tag_instances()




import os
from datetime import datetime, timedelta
from typing import Union


class Utils:

    def __init__(self):
        pass

    @staticmethod
    def get_cloud_policies(cloud_name: str, file_type: str = '.py', dir_dict: bool = False,
                           exclude_policies: list = None):
        """
        This method returns the policies by cloud_name
        :return:
        :rtype:
        """
        cloud_name = cloud_name.lower()
        exclude_policies = [] if not exclude_policies else exclude_policies
        policies_dict = {}
        policies_list = []
        policies_path = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(__file__))), 'policy', cloud_name)
        for (dir_path, _, filenames) in os.walk(policies_path):
            immediate_parent = dir_path.split("/")[-1]
            for filename in filenames:
                if not filename.startswith('__') and filename.endswith(file_type):
                    filename = os.path.splitext(filename)[0]
                    if filename not in exclude_policies:
                        if dir_dict:
                            policies_dict.setdefault(immediate_parent, []).append(filename)
                        else:
                            policies_list.append(filename)
        return policies_dict if dir_dict else policies_list

    @staticmethod
    def equal_ignore_case(str1: str, str2: str, *args):
        """
        This method returns boolean by comparing equal in-case sensitive all strings
        :param str1:
        :type str1:
        :param str2:
        :type str2:
        :param args:
        :type args:
        :return:
        :rtype:
        """
        equal = str1.lower() == str2.lower()
        for val in args:
            equal = str1.lower() == val.lower() and equal
        return equal

    @staticmethod
    def contains_ignore_case(string: str, str1: str):
        """
        This method check that
        :param string:
        :type string:
        :param str1:
        :type str1:
        :return:
        :rtype:
        """
        return str1.lower() in string

    @staticmethod
    def greater_than(val1: Union[int, float], val2: Union[int, float]) -> bool:
        """
        This method returns bool, on performing the greater on val1 to val2
        :param val1:
        :type val1:
        :param val2:
        :type val2:
        :return:
        :rtype:
        """
        return val1 > val2

    @staticmethod
    def greater_than_equal(val1: Union[int, float], val2: Union[int, float]) -> bool:
        """
        This method returns bool, on performing the greater equal on val1 to val2
        :param val1:
        :type val1:
        :param val2:
        :type val2:
        :return:
        :rtype:
        """
        return val1 >= val2

    @staticmethod
    def less_than(val1: Union[int, float], val2: Union[int, float]):
        """
        This method returns bool, on performing the less than operation on val1 to val2
        :param val1:
        :type val1:
        :param val2:
        :type val2:
        :return:
        :rtype:
        """
        return val1 < val2

    @staticmethod
    def less_than_equal(val1: Union[int, float], val2: Union[int, float]):
        """
        This method returns bool, on performing the less than equal operation on val1 to val2
        :param val1:
        :type val1:
        :param val2:
        :type val2:
        :return:
        :rtype:
        """
        return val1 <= val2

    @staticmethod
    def get_start_and_end_datetime(days: int) -> [datetime, datetime]:
        """
        This method returns the start and end datetime
        :param days:
        :type days:
        :return:
        :rtype:
        """
        days = 1 if days == 0 else days
        end_date = datetime.utcnow()
        start_date = end_date - timedelta(days=days)
        return start_date, end_date



LOOK_BACK_DAYS = 30
MONTHS = 12
DEFAULT_ROUND_DIGITS = 3
PUBLIC_ACCOUNTS_COST_REPORTS = 'Accounts'
HOURS_IN_SECONDS = 3600
HOURS_IN_DAY = 24
HOURS_IN_MONTH = 720
TOTAL_BYTES_IN_KIB = 1024

DATE_FORMAT = "%Y-%m-%d"
DATE_TIME_FORMAT_T = "%Y-%m-%dT%h:%m"
UNUSED_DAYS = 7

# Policy Values
# Instance Idle
INSTANCE_IDLE_DAYS = 7
INSTANCE_IDLE_CPU_PERCENTAGE = 2
INSTANCE_IDLE_NETWORK_IN_KILO_BYTES = 5  # In KiB
INSTANCE_IDLE_NETWORK_OUT_KILO_BYTES = 5  # In KiB
EC2_NAMESPACE = 'AWS/EC2'
CLOUDWATCH_METRICS_AVAILABLE_DAYS = 14
AWS_DEFAULT_GLOBAL_REGION = 'us-east-1'
